from .term import TermClassifier
from .math import BotMath
import pandas as pd

class BotBody(BotMath, TermClassifier):

    def __init__(self):
        self.questions = []
        self.answers = []
        self.score_value = 0

    def fit(self, data = ""):
        """
        fit training data
        """
        data = pd.read_csv(data)
        self.questions = data.iloc[:, 0]
        self.answers = data.iloc[:, -1]

    def predict(self, questions = ''):
        """
        predict new questions used for, ask the bot about the answer
        """
        pred_data =  self.questions.append(pd.Series([questions]), ignore_index=True)

        # get the answer
        return self.__get_answer(questions=pred_data)

    def __get_answer(self, questions = []):
        """
        get the answer by calculating text similarity based on tf-idf
        """

        vector = TermClassifier()
        t_questions = self._tfidf(questions.values)

        # calculate similarity
        similarity = []
        for i in range(0, len(t_questions) - 1):
            similarity.append(self._similarity(t_questions[len(t_questions) - 1], t_questions[i]))
        
        # get index array of max value similarity
        index = similarity.index(max(similarity))

        # return the answer
        self.score_value = max(similarity)
        return self.answers.values[index]

    def score(self):
        """
        return the similarity score
        """

        return self.score_value


import numpy as np

class BotMath:

    def _similarity(self, x = [], y = []):
        """
        calculate similarity between 2 vectors
        """
        a = np.array(x)
        b = np.array(y)
        
        penyebut = np.sum(a * b)
        pembilang = np.sqrt(np.sum(a**2)) * np.sqrt(np.sum(b**2))

        return penyebut / pembilang


import numpy as np
import re
import string

class TermClassifier:

    def _tfidf(self, data = []):
        """
        term weighting
        """
        doc = []

        # remove punctuation
        for text in data:
            doc.append(self.__remove_punctuation(text).lower())

        # extract unique features

        bow = []
        features = ''
        for i in doc:
            for word in i.split(" "):
                features += word + " "
                if word not in bow and word != '':
                    bow.append(word)

        
        # calculate tfidf
        matrix = []
        for document in doc:
            arr = []
            for word in bow:
                tf = len(re.findall(word, document)) / len(document.split(" "))
                idf = np.log(len(doc) / len(re.findall(word, features)))
                arr.append(tf * idf)
            matrix.append(arr)

        return np.array(matrix)

    def __remove_punctuation(self, words):
        """
        remove punctuation ?~!@#$%^
        """
        
        clean_str = words.lower()
        for pct in string.punctuation:
            clean_str = clean_str.replace(pct, "")

        # remove space on the last words
        return clean_str


from setuptools import setup

def readme():
    readme = ''
    with open("README.md") as files:
        readme = files.read()
    return readme

setup(
  name = 'iokobot',
  packages = ['iokobot'],
  version = '0.4',
  license='MIT',
  description = 'iokobot is a python library to help you make a smart bot, using machine learning algorithm',
  long_description=readme(),
  long_description_content_type="text/markdown",
  author = 'Rizki Maulana',
  author_email = 'rizkimaulana348@gmail.com',
  url = 'https://github.com/rizki4106/iokobot',
  download_url = 'https://github.com/rizki4106/iokobot/archive/v_01.tar.gz',
  keywords = ['bot', 'chatbot', 'iokobot', 'machine-learning'],
  install_requires = [
      'scikit-learn',
      'numpy',
      'pandas'
  ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
  ]
)